//+------------------------------------------------------------------+
//|                                               FrameGenerator.mqh |
//|                        Copyright 2012, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2012, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

//---         
#include <SpecialChart.mqh>
#include <SimpleTable.mqh>
#include <ColorProgressBar.mqh>
#include <Controls\Button.mqh>
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
class CFrameGenerator
  {
private:
   int               m_trades;
   int               m_deals;
   double            m_profit;
   double            m_lots;
   double            m_ddawn_money;
   double            m_ddown_percent;
   double            m_profitfactor;
   double            m_recoveryfactor;
   CSpecialChart     m_spec_chart;
   CSimpleTable      m_statistics;
   CSimpleTable      m_inputs;
   CColorProgressBar m_cpb;
   CButton           m_replaybutton;
   bool              m_completed;
public:
   //--- /
                     CFrameGenerator();
                    ~CFrameGenerator();
   //---   
   void              OnTester(const double OnTesterValue);
   void              OnTesterInit(int lines);
   void              OnTesterPass(void);
   void              OnTesterDeinit(void);
   //---  
   void              OnChartEvent(const int id,const long &lparam,const double &dparam,const string &sparam,const int delay_ms);
   //---  
   void              ReplayFrames(const int delay_ms);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CFrameGenerator::CFrameGenerator()
  {
   m_completed=false;
   Print(__FUNCTION__);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CFrameGenerator::~CFrameGenerator()
  {
  }
//+------------------------------------------------------------------+
//|      OnTesterInit()                  | 
//+------------------------------------------------------------------+
void CFrameGenerator::OnTesterInit(int lines)
  {
   Comment("Waiting for initialization of optimization...");
//---        
   if(!m_spec_chart.CreateBitmapLabel("Chart",200,20,500,260,COLOR_FORMAT_XRGB_NOALPHA))
      Print(" Create()  false");
//---     
   m_spec_chart.SetBackground(clrGray);
   m_spec_chart.SetFrameColor(clrIvory);
   m_spec_chart.SetFrameWidth(1);
//---    
   m_spec_chart.SetLines(lines);
//--- 
   m_spec_chart.Update();
   Comment(" ");
//---   
   m_cpb.Create("color passes",200,260,500,20,COLOR_FORMAT_XRGB_NOALPHA);
   m_cpb.BackColor(clrIvory);
   m_cpb.BorderColor(clrGray);
   m_cpb.BorderWidth(1);
   m_cpb.Update();
//---   
   m_statistics.Create("statistics",0,20,135,65,20);
   m_statistics.BackColor(clrWhite);
   m_statistics.BorderColor(clrGray);
   m_statistics.TextColor(clrBlack);
   m_statistics.AddRow("Net Profit"," - ");
   m_statistics.AddRow("Profit Factor"," - ");
   m_statistics.AddRow("Factor Recovery"," - ");
   m_statistics.AddRow("Trades"," - ");
   m_statistics.AddRow("Deals "," - ");
   m_statistics.AddRow("Equity DD"," - ");
   m_statistics.AddRow("OnTester()"," - ");
//---    
   m_inputs.Create("statistics",0,158,135,65,20);
   m_inputs.BackColor(clrMintCream);
   m_inputs.BorderColor(clrGray);
   m_inputs.TextColor(clrBlack);
//---
   m_replaybutton.Create(0,"repaly button",0,200,20,0,0);
   m_replaybutton.Width(500);
   m_replaybutton.Height(20);
   m_replaybutton.Text("");
//---    
   ChartRedraw();
   Print(__FUNCTION__);
  }
//+------------------------------------------------------------------+
//|      OnTesterDeinit()                | 
//+------------------------------------------------------------------+
void CFrameGenerator::OnTesterDeinit(void)
  {
   m_completed=true;
   Comment("Optimization completed");
//---      
   m_replaybutton.ColorBackground(clrLightGreen);
   m_replaybutton.Text(" :    ");
  }
//+------------------------------------------------------------------+
//|                |
//|          OnTester()  |
//+------------------------------------------------------------------+
void CFrameGenerator::OnTester(const double OnTesterValue)
  {
//---
   double   balance[];
   int      data_count=0;
   double   balance_current=1000;
//---      
   ulong    ticket=0;
   double   profit;
   string   symbol;
   long     entry;
//---    
   HistorySelect(0,TimeCurrent());
   uint deals_total=HistoryDealsTotal();
//---    
   for(uint i=0;i<deals_total;i++)
     {
      if((ticket=HistoryDealGetTicket(i))>0)
        {
         symbol=HistoryDealGetString(ticket,DEAL_SYMBOL);
         entry =HistoryDealGetInteger(ticket,DEAL_ENTRY);
         profit=HistoryDealGetDouble(ticket,DEAL_PROFIT);
         if(entry==DEAL_ENTRY_OUT || entry==DEAL_ENTRY_INOUT)
           {
            balance_current+=profit;
            data_count++;
            ArrayResize(balance,data_count);
            balance[data_count-1]=balance_current;
           }
        }
     }
//---  data[]     
   double data[];
   ArrayResize(data,ArraySize(balance)+7);
   ArrayCopy(data,balance,7,0);
//---   7    
   data[0]=TesterStatistics(STAT_PROFIT);                //  
   data[1]=TesterStatistics(STAT_PROFIT_FACTOR);         //  
   data[2]=TesterStatistics(STAT_RECOVERY_FACTOR);       //  
   data[3]=TesterStatistics(STAT_TRADES);                //  
   data[4]=TesterStatistics(STAT_DEALS);                 //  
   data[5]=TesterStatistics(STAT_EQUITY_DDREL_PERCENT);  //     
   data[6]=OnTesterValue;                                //    
//---         ( )
   PrintFormat("STAT_PROFIT=%.2f",data[0]);
   PrintFormat("STAT_PROFIT_FACTOR=%.2f",data[1]);
   PrintFormat("STAT_RECOVERY_FACTOR=%.2f",data[2]);
   PrintFormat("STAT_TRADES=%d",(int)data[3]);
   PrintFormat("STAT_DEALS=%d",(int)data[4]);
   PrintFormat("STAT_EQUITY_DDREL_PERCENT=%.2f%%",data[5]);
   PrintFormat("STAT_CUSTOM_ONTESTER=%G",data[6]);
//---         
   if(!FrameAdd(MQL5InfoString(MQL5_PROGRAM_NAME),1,deals_total,data))
      Print("Frame add error: ",GetLastError());
   else
      Print("Frame added, Ok");
//---
  }
//+------------------------------------------------------------------+
//|              |
//+------------------------------------------------------------------+
void CFrameGenerator::OnTesterPass(void)
  {
//---     
   string  name;
   ulong   pass;
   long    id;
   double  value,data[];
   string params[];
   uint par_count;
//---  
   static datetime start=0;
   static int      cnt=0;
//---
   if(start==0) start=TimeLocal();
//---         
   while(FrameNext(pass,name,id,value,data))
     {
      //---   
      string comm="OnTesterPass("+name+"), pass="+(string)pass+" id="+(string)id+
                  "  Deals="+StringFormat("%d",(int)value)+"\r\n";
      Comment(comm);
      //---     
      if(m_inputs.Rows()==0)
        {
         //---    ,    
         if(FrameInputs(pass,params,par_count))
           {
            //---  , params[i]   "parameter=value"
            for(uint i=0;i<par_count;i++)
              {
               //---    
               string array[];
               //---         
               if(StringSplit(params[i],'=',array)==2) m_inputs.AddRow(array[0],array[1]);
              }
           }
        }
      else //---   ,        
        {
         //---    ,    
         if(FrameInputs(pass,params,par_count))
           {
            //---  , params[i]   "parameter=value"
            for(uint i=0;i<par_count;i++)
              {
               //---   
               StringAdd(comm,params[i]+"\r\n");
               //---    
               string array[];
               //---          
               if(StringSplit(params[i],'=',array)==2) m_inputs.SetValue(1,i,array[1]);
              }
           }

        }
      //---   
      m_statistics.SetValue(1,0,StringFormat("%.2f",data[0]));
      m_statistics.SetValue(1,1,StringFormat("%.2f",data[1]));
      m_statistics.SetValue(1,2,StringFormat("%.2f",data[2]));
      m_statistics.SetValue(1,3,StringFormat("%G",data[3]));
      m_statistics.SetValue(1,4,StringFormat("%G",data[4]));
      m_statistics.SetValue(1,5,StringFormat("%.2f%%",data[5]));
      m_statistics.SetValue(1,6,StringFormat("%G",data[6]));

      //---       
      double seria[];
      ArrayCopy(seria,data,0,7,ArraySize(data)-7);
      //---        
      m_spec_chart.AddSeria(seria,data[0]>0);
      //---      
      m_spec_chart.Update();
      //---   
      m_cpb.AddResult(data[0]>0);
      //---    
      cnt++;
      //---     
      m_replaybutton.Text(StringFormat("  ( ): %d  %s",
                          cnt,TimeToString(TimeLocal()-start,TIME_MINUTES|TIME_SECONDS)));
     }
//---
  }
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
void CFrameGenerator::OnChartEvent(const int id,const long &lparam,
                                   const double &dparam,const string &sparam,
                                   const int delay_ms)
  {
   if(!m_completed) return;
//---        
   if(id==CHARTEVENT_OBJECT_CLICK)
     {
      //---     
      if(sparam==m_replaybutton.Name())
        {
         //---  
         ObjectSetInteger(0,sparam,OBJPROP_STATE,false);
         //---   
         m_replaybutton.ColorBackground(clrKhaki);
         //---  
         m_completed=false; //      
         ReplayFrames(100); //   
         m_completed=true; //     
         //---     
         m_replaybutton.ColorBackground(clrLightGreen);
         m_replaybutton.Text(" :    ");
        }
     }
  }
//+------------------------------------------------------------------+
//|             |
//+------------------------------------------------------------------+
void CFrameGenerator::ReplayFrames(const int delay_ms)
  {
//---     
   string  name;
   ulong   pass;
   long    id;
   double  value,data[];
   string params[];
   uint par_count;
//---  
   int      frame_counter=0;
//---   -
   m_cpb.Reset();
   m_cpb.Update();
//---    
   FrameFirst();
//---   
   while(FrameNext(pass,name,id,value,data))
     {
      //---    ,    
      if(FrameInputs(pass,params,par_count))
        {
         //---  , params[i]   "parameter=value"
         for(uint i=0;i<par_count;i++)
           {
            //---    
            string array[];
            //---          
            if(StringSplit(params[i],'=',array)==2) m_inputs.SetValue(1,i,array[1]);
           }
        }
      //---   
      m_statistics.SetValue(1,0,StringFormat("%.2f",data[0]));
      m_statistics.SetValue(1,1,StringFormat("%.2f",data[1]));
      m_statistics.SetValue(1,2,StringFormat("%.2f",data[2]));
      m_statistics.SetValue(1,3,StringFormat("%G",data[3]));
      m_statistics.SetValue(1,4,StringFormat("%G",data[4]));
      m_statistics.SetValue(1,5,StringFormat("%.2f%%",data[5]));
      m_statistics.SetValue(1,6,StringFormat("%G",data[6]));
      //---       
      double seria[];
      ArrayCopy(seria,data,0,7,ArraySize(data)-7);
      //---        
      m_spec_chart.AddSeria(seria,data[0]>0);
      //---      
      m_spec_chart.Update();
      //---   
      m_cpb.AddResult(data[0]>0);
      //---    
      frame_counter++;
      //---     
      m_replaybutton.Text(StringFormat("   %d ms:  %d",
                          delay_ms,frame_counter));
      Sleep(delay_ms);
     }
  }

//+------------------------------------------------------------------+
